//FadenGrafik_301d.js
//Eine Erweiterung des Scripts "MysticRose.jsx" von Olav Kvern
// 12.10.10 / Hans Haesler, Chtelard 52, CH-1018 Lausanne / <hsa@ringier.ch>
//DESCRIPTION:Zeichnet eine "mystic rose" (eine geometrische Konstruktion, welche auch Fadengrafik genannt wird)

// vorsichtshalber die Dialoge einschalten
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;

// "Keine" Farbe und "Passkreuze" je nach Version
var noColor = "[Ohne]";
var rgColor = "[Passermarken]";
if (app.version[0] == 4) {
	noColor = "[Keine]";
	rgColor = "[Passkreuze]";
}

if (app.documents.length!=0) {
	if (app.activeWindow.constructor.name == "LayoutWindow") {
		var curDoc = app.documents[0];
		// den Handler aufrufen, welcher die gespeicherten Vorgaben einliest
		var allPrefs = getPrefs();
		if (allPrefs != "") {
			var prefVal = allPrefs.split("\n");
			var xPos  = prefVal[0]*1;
			var yPos  = prefVal[1]*1;
			var curD  = prefVal[2]*1;
			var nPts  = prefVal[3]*1;
			var bName = prefVal[4];
			if (bName == "Black") { bName = "[Schwarz]"; }
			else if (bName == "Paper") { bName = "[Papier]"; }
			else if (bName == "Registration") { bName = rgColor; }
			else if (bName == "None") { bName = noColor; }
			var bgrT  = prefVal[5]*1;
			var ck0   = true;
			var eoFll = prefVal[6];
			if (eoFll == "false") {
				ck0 = false;
			}
			var sName = prefVal[7];
			if (sName == "Black") { sName = "[Schwarz]"; }
			else if (sName == "Paper") { sName = "[Papier]"; }
			else if (sName == "Registration") { sName = rgColor; }
			else if (sName == "None") { sName = noColor; }
			var strT  = prefVal[8]*1;
			var strW  = prefVal[9]*1;
			var alig  = prefVal[10];
			var ck1 = 1;
			var ck2 = 0;
			var ck3 = 0;
			if (alig == "1") {
				ck1 = 0;
				ck2 = 1;
			}
			else if (alig == "2") {
				ck1 = 0;
				ck3 = 1;
			}
		}
		else {
			// wenn die Prferenzen-Datei nicht gefunden wurde
			// dann auf die Vorgabenwerte zurckgreifen
			var xPos =  100;
			var yPos =  100;
			var curD =  100;
			var nPts =   21;
			var bgrT =   70;
			var ck0  = true;
			var strT =  100;
			var strW = 0.25;
			var ck1  =    1;
			var ck2  =    0;
			var ck3  =    0;
			var bName = "C=0 M=100 Y=0 K=0";
			var sName = "[Schwarz]";
		}
		
		// die Handler aufrufen, welche die Listen der Farben vorbereiten
		var bgrColor = getBgrColorList(bName);
		var bgrCol   = bgrColor[0];
		var bInd     = bgrColor[1];
		var strColor = getStrColorList(sName);
		var strCol   = strColor[0];
		var sInd     = strColor[1];
		
		// die Masseinheiten speichern
		// wenn nicht 2053991795 (= Millimeter), dann auf Millimeter umstellen
		hM = curDoc.viewPreferences.horizontalMeasurementUnits;
		vM = curDoc.viewPreferences.verticalMeasurementUnits;
		if (hM != 2053991795) {
			curDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.millimeters;
		}
		if (vM != 2053991795) {
			curDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.millimeters;
		}
		// den Nullpunkt auf Null/Null setzen
		var zP = curDoc.zeroPoint; 
		if (zP != [0,0]) { 
			curDoc.zeroPoint = [0,0]; 
		}
		
		// den Handler aufrufen, welcher den Dialog zeigt und auswertet
		var anArray = displayDialog(xPos, yPos, curD, nPts, bgrCol, bInd, bgrT, ck0, strCol, sInd, strT, strW, ck1, ck2, ck3);
		var centerPoint = anArray[0];
		var curDiameter = anArray[1];
		var nPoints     = anArray[2];
		var bColor      = anArray[3];
		var bgrT        = anArray[4];
		var evenOddFill = anArray[5];
		var sColor      = anArray[6];
		var strT        = anArray[7];
		var strW        = anArray[8];
		var strA        = anArray[9];
		// den Handler aufrufen, welcher die Polygone berechnet
		var curPathPoints = calculatePolygon(centerPoint, curDiameter/2, nPoints);
		// den Handler aufrufen, welcher die Rose zeichnet
		drawMagicRose(curPathPoints, nPoints, bColor, bgrT, evenOddFill, sColor, strT, strW, strA);
		
		// den Nullpunkt zurcksetzen (falls notwendig) 
		if (zP != [0,0]) { 
			curDoc.zeroPoint = zP; 
		}
		
		// Masseinheiten zurcksetzen (falls notwendig)
		if (hM != 2053991795) {
			curDoc.viewPreferences.horizontalMeasurementUnits = hM;
		}
		if (vM != 2053991795) {
			curDoc.viewPreferences.verticalMeasurementUnits = vM;
		}
	}
}

// der Handler, welcher die Informationen vorbereitet
function getPrefs() {
	var allPrefs = "";
	// die Datei lesen, welche die gespeicherten Werte enthlt
	var tempFolder = Folder.temp;
	var f = File(tempFolder + "/faden.txt");
	var fileIsOpen = f.open("r");
	if (fileIsOpen) {
		try {
			allPrefs = f.read();
			f.close();
		}
		catch (e) {
			try {
				f.close();
			}
			catch (e) {
			}
		}
	}
	return allPrefs;
}

// der Handler, welcher die Liste der Hintergrund-Farben vorbereitet
function getBgrColorList(bName) {
	// eine Variable als Referenz zu den Farbfeldern des aktiven Dokuments definieren
	var allSwatches = curDoc.swatches;
	
	// prfen, ob die Vorgabefarbe existiert
	if (bName != "[Schwarz]" && bName != "[Papier]" && bName != noColor && bName != rgColor) {
		try {
			curDoc.swatches.itemByName(bName).name;
		}
		catch (e) {
			bName = "[Schwarz]";
		}
	}
	
	// die Liste der Farben zusammenstellen;
	// "None", "Paper", "Black" und "Registration" umbenennen;
	// die Position der Vorgabefarbe feststellen
	var bgrColorList = new Array();
	for (var i=0; i<allSwatches.length; i++) {
		cName = allSwatches.item(i).name;
		if (cName == "None") {
			cName = noColor;
		}
		else if (cName == "Paper") {
			cName = "[Papier]";
		}
		else if (cName == "Black") {
			cName = "[Schwarz]";
		}
		else if (cName == "Registration") {
			cName = rgColor;
		}
		if (bName == cName) {
			bIndex = i;
		}
		if (cName != "Registration") {
			bgrColorList.push(cName);
		}
	}
	return [bgrColorList, bIndex];
}

// der Handler, welcher die Liste der Kontur-Farben vorbereitet
function getStrColorList(sName) {
	// eine Variable als Referenz zu den Farbfeldern des aktiven Dokuments definieren
	var allSwatches = curDoc.swatches;
	
	// prfen, ob die Vorgabefarbe existiert
	if (sName != "[Schwarz]" && sName != "[Papier]" && sName != noColor && sName != rgColor) {
		try {
			curDoc.swatches.itemByName(sName).name;
		}
		catch (e) {
			sName = "[Schwarz]";
		}
	}
	
	// die Liste der Farben zusammenstellen;
	// "None", "Paper", "Black" und "Registration" umbenennen;
	// die Position der Vorgabefarbe feststellen
	var strColorList = new Array();
	for (var i=0; i<allSwatches.length; i++) {
		cName = allSwatches.item(i).name;
		if (cName == "None") {
			cName = noColor;
		}
		else if (cName == "Paper") {
			cName = "[Papier]";
		}
		else if (cName == "Black") {
			cName = "[Schwarz]";
		}
		else if (cName == "Registration") {
			cName = rgColor;
		}
		if (sName == cName) {
			sIndex = i;
		}
		if (cName != "Registration") {
			strColorList.push(cName);
		}
	}
	return [strColorList, sIndex];
}

// den Dialog vorbereiten und anzeigen
function displayDialog(xPos, yPos, curD, nPts, bgrCol, bInd, bgrT, ck0, strCol, sInd, strT, strW, ck1, ck2, ck3) {
	try {
		app.dialogs.everyItem().destroy();
	}
	catch (e) {
	}
	var labelWidth = 70;
	var fieldWidth = 40;
	var aDialog = app.dialogs.add({name:"FadenGrafik"});
	with (aDialog.dialogColumns.add()) {
		with (dialogRows.add()) {
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"Zentrum", minWidth:labelWidth});
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"X:"});
			}
			with (dialogColumns.add()) {
				var xField = realEditboxes.add({editValue:xPos, minWidth:fieldWidth});
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"mm"});
			}
		}
		with (dialogRows.add()) {
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"", minWidth:labelWidth});
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"Y:"});
			}
			with (dialogColumns.add()) {
				var yField = realEditboxes.add({editValue:yPos, minWidth:fieldWidth});
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"mm"});
			}
		}
		labelWidth = 95;
		with (dialogRows.add()) {
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"Durchmesser", minWidth:labelWidth});
			}
			with (dialogColumns.add()) {
				var curDiameterField = realEditboxes.add({editValue:curD, minWidth:fieldWidth});
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"mm"});
			}
		}
		with (dialogRows.add()) {
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"Anzahl Punkte", minWidth:labelWidth});
			}
			with (dialogColumns.add()) {
				var nPointsField = integerEditboxes.add({editValue:nPts, minimumValue:5, maximumValue:39, minWidth:fieldWidth});
			}
		}
		with (borderPanels.add()) {
			with (dialogColumns.add()) {
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Hintergrund", minWidth:labelWidth-10});
					staticTexts.add({staticLabel:" "});
				}
				var bgrColor = dropdowns.add({stringList:bgrCol, selectedIndex:bInd, minWidth:130});
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Tonwert", minWidth:labelWidth-10});
					var bgrTint = realEditboxes.add({editValue:bgrT, minimumValue:0, maximumValue:100, minWidth:fieldWidth});
					staticTexts.add({staticLabel:"%", minWidth:12});
				}
			}
		}
		var evenOddCheckbox = checkboxControls.add({staticLabel:"Abwechselnd fllen", checkedState:ck0});
		with (borderPanels.add()) {
			with (dialogColumns.add()) {
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Kontur", minWidth:labelWidth-10});
					staticTexts.add({staticLabel:" "});
				}
				var strColor = dropdowns.add({stringList:strCol, selectedIndex:sInd, minWidth:130});
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Tonwert", minWidth:labelWidth-10});
					var strTint = realEditboxes.add({editValue:strT, minimumValue:0, maximumValue:100, minWidth:fieldWidth});
					staticTexts.add({staticLabel:"%", minWidth:12});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Konturstrke", minWidth:labelWidth-10});
					var strWeight = realEditboxes.add({editValue:strW, minimumValue:0, maximumValue:5, minWidth:fieldWidth});
					staticTexts.add({staticLabel:"pt", minWidth:12});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Ausrichtung", minWidth:labelWidth-10});
					with(aligButtons = radiobuttonGroups.add()){
						radiobuttonControls.add({staticLabel:"mittig", checkedState:ck1});
						radiobuttonControls.add({staticLabel:"innen", checkedState:ck2});
						radiobuttonControls.add({staticLabel:"aussen", checkedState:ck3});
					}
				}
			}
		}
	}
	if(aDialog.show() == true) {
		var x = xField.editValue;
		var y = yField.editValue;
		var curDiameter = curDiameterField.editValue;
		var nPoints = nPointsField.editValue;
		if ((nPoints%2) == 0) {
			alert ("Die Anzahl Punkte sollte *ungerade* sein.", "Achtung");
		}
		bgrIndex = bgrColor.selectedIndex;
		bColor = curDoc.swatches[bgrIndex].name;
		if (bColor == "[Schwarz]") { bColor = "Black"; }
		else if (bColor == "[Papier]") { bColor = "Paper"; }
		else if (bColor == rgColor) { bColor = "Registration"; }
		else if (bColor == noColor) { bColor = "None"; }
		bgrT = bgrTint.editValue;
		var ck0 = evenOddCheckbox.checkedState;
		strIndex = strColor.selectedIndex;
		sColor = curDoc.swatches[strIndex].name;
		if (sColor == "[Schwarz]") { sColor = "Black"; }
		else if (sColor == "[Papier]") { sColor = "Paper"; }
		else if (sColor == rgColor) { sColor = "Registration"; }
		else if (sColor == noColor) { sColor = "None"; }
		strT = strTint.editValue;
		strW = strWeight.editValue;
		var aligDec = aligButtons.selectedButton;
		if (aligDec == 0) {
			strA = 1936998723; // mittig
		}
		else if (aligDec == 1) {
			strA = 1936998729; // innen
		}
		else if (aligDec == 2) {
			strA = 1936998735; // aussen
		}
		aDialog.destroy();
		var prefsList = new Array (x, y, curDiameter, nPoints, bColor, bgrT, ck0, sColor, strT, strW, aligDec);
		var prefString = prefsList.join("\n");
		writeResult(prefString);
		return [[x, y], curDiameter, nPoints, bColor, bgrT, ck0, sColor, strT, strW, strA];
	}
	else{
		aDialog.destroy();
		exit();
	}
}

// das Ergebnis in eine Datei schreiben
function writeResult(curPrefs) {
	var tempFolder = Folder.temp;
	var f = File(tempFolder + "/faden.txt");
	var fileIsOpen = f.open("w");
	if (fileIsOpen) {
		try {
			f.write(curPrefs);
			f.close();
		}
		catch (e) {
			try {
				f.close();
			}
			catch (e) {
			}
		}
	}
}

// diese Funktion errechnet die Punkte fr ein Polygon mit n-Seiten um ein Zentrum herum
function calculatePolygon(centerPoint, curRadius, nPoints) {
	angleIncrement = (360/nPoints)*(Math.PI/180);
	var curPathPoints = new Array;
	for (pointCounter = 0; pointCounter < nPoints;pointCounter ++) {
		x = centerPoint[0] + (curRadius * Math.cos(angleIncrement*pointCounter));
		y = centerPoint[1] + (curRadius * Math.sin(angleIncrement*pointCounter));
		curPathPoints.push([x, y]);
	}
	return curPathPoints;
}

// diese Funktion zeichnet die Rose
function drawMagicRose(curPathPoints, nPoints, bColor, bgrT, ck0, sColor, strT, strW, strA) {
	// die zu berspringenden Punkte auflisten
	var skipArray = new Array ();
	var nVals = Math.floor(nPoints/2)+1;
	for (var v=1; v<nVals; v++) {
		skipArray.push(v);
	}
	// weitere Variablen definieren
	var isDone = false;
	var ctr = 1;
	var curIndex = 0;
	var newPathPoints = new Array;
	newPathPoints.push(curPathPoints[0]);
	do {
		for (var skipCounter = 0; skipCounter < skipArray.length; skipCounter++) {
			var curIndex = (curIndex + skipArray[skipCounter])%curPathPoints.length;
			newPathPoints.push(curPathPoints[curIndex]);
		}
		if (curIndex == 0) {
			// wenn der Ausgangspunkt am Ende der Sequenz erreicht ist, dann ist Schluss
			// N.B.: Der Ausgangspunkt kann *innerhalb* der Schritt-Sequenz viele Male
			// erreicht werden
			isDone = true;
		}
		else {
			ctr ++;
		}
	} while (isDone != true);
	bColor = curDoc.swatches.itemByName(bColor);
	sColor = curDoc.swatches.itemByName(sColor);
	var curPolygon = app.activeWindow.activePage.polygons.add();
	curPolygon.paths.item(0).entirePath = newPathPoints;
	if (ck0 == true) {
		var newPolygon = curPolygon.duplicate();
		curPolygon.addPath(newPolygon);
		newPolygon.properties = {fillColor:bColor, fillTint:bgrT, strokeWeight:strW, strokeColor:sColor, strokeTint:strT, strokeAlignment:strA};
	}
	else {
		curPolygon.properties = {fillColor:bColor, fillTint:bgrT, strokeWeight:strW, strokeColor:sColor, strokeTint:strT, strokeAlignment:strA};
	}
}